import 'package:easy_validation/annotations/validation_annotation.dart';
import 'package:easy_validation/annotations/validation_helper.dart';
import 'package:flutter/widgets.dart';

/// [SmallerOrEqualTo] validator permits you to check that a value is smaller or equal to the value of another property.
/// {@category Metadata}
/// {@subCategory Validators}
class SmallerOrEqualTo extends ValidationAnnotation {
  const SmallerOrEqualTo({
    @required this.propertyName,
    @required this.error,
  }) : super(criticityLevel: 2, error: error);

  /// This is the name of targeted property.
  final String propertyName;

  /// This is the custom error to return in case of invalidation.
  final String error;

  @override
  bool isValid<TModel>(Object value, TModel model) {
    try {
      Object valueToCompare =
          ValidationHelper.getLinkedProperty(model, this.propertyName);
      bool defaultValidation =
          ValidationHelper.getDefaultValidation(value, valueToCompare);

      if (defaultValidation != null) return defaultValidation;

      if (value is DateTime)
        return _isValidDatetime(value, valueToCompare);
      else if (value is int)
        return _isValidNumber(value.toDouble(), valueToCompare);
      else if (value is double)
        return _isValidNumber(value, valueToCompare);
      else
        throw Exception('Can\'t convert value to number or datetime');
    } catch (e) {
      print(e);
      return false;
    }
  }

  bool _isValidDatetime(DateTime value, Object valueToCompare) {
    if (valueToCompare is DateTime) {
      if (value.isBefore(valueToCompare) ||
          (value.compareTo(valueToCompare) == 0)) return true;
      return false;
    } else
      throw Exception('Comparison field is not a datetime');
  }

  bool _isValidNumber(double value, Object valueToCompare) {
    if (valueToCompare is int || valueToCompare is double) {
      if (value <= valueToCompare) return true;
      return false;
    } else
      throw Exception('Comparison field is not a number');
  }
}
