import 'package:easy_validation/easy_validation.dart';
import 'package:flutter_test/flutter_test.dart';
import 'easy_validation_test.reflectable.dart';

void main() {
  initializeReflectable();

  /*****************************************
   * Equal to
  *****************************************/
  test('Test for EqualTo is a success. Model is valid.', () {
    EqualToTest tester = new EqualToTest(DateTime.now(), DateTime.now(), 1, 1, 2.0, 2.0, 'azerty', 'azerty');
    bool isValid = ModelState.isValid<EqualToTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for EqualTo is a success. Model is not valid.', () {
    EqualToTest tester = new EqualToTest(DateTime.now(), DateTime.parse('2019-01-01'), 1, 2, 2.0, 2.5, 'azerty', 'ytreza');
    bool isValid = ModelState.isValid<EqualToTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, EqualTo);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être égale à la propriété "b"');
    expect(ModelState.errors[1].validatorType, EqualTo);
    expect(ModelState.errors[1].propertyName, 'c');
    expect(ModelState.errors[1].error, 'La propriété "c" doit être égale à la propriété "d"');
    expect(ModelState.errors[2].validatorType, EqualTo);
    expect(ModelState.errors[2].propertyName, 'e');
    expect(ModelState.errors[2].error, 'La propriété "e" doit être égale à la propriété "f"');
    expect(ModelState.errors[3].validatorType, EqualTo);
    expect(ModelState.errors[3].propertyName, 'g');
    expect(ModelState.errors[3].error, 'La propriété "g" doit être égale à la propriété "h"');
  });

  /*****************************************
   * Greater or equal to
  *****************************************/
  test('Test for GreaterOrEqualTo is a success. Model is valid.', () {
    GreaterOrEqualToTest tester = new GreaterOrEqualToTest(DateTime.parse('2019-01-10'), DateTime.parse('2019-01-09'), 5, 5, 7.5, 7.0);
    bool isValid = ModelState.isValid<GreaterOrEqualToTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for GreaterOrEqualTo is a success. Model is not valid.', () {
    GreaterOrEqualToTest tester = new GreaterOrEqualToTest(DateTime.parse('2019-01-09'), DateTime.parse('2019-01-10'), 5, 6, 7.0, 7.1);
    bool isValid = ModelState.isValid<GreaterOrEqualToTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, GreaterOrEqualTo);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être supérieure ou égale à la propriété "b"');
    expect(ModelState.errors[1].validatorType, GreaterOrEqualTo);
    expect(ModelState.errors[1].propertyName, 'c');
    expect(ModelState.errors[1].error, 'La propriété "c" doit être supérieure ou égale à la propriété "d"');
    expect(ModelState.errors[2].validatorType, GreaterOrEqualTo);
    expect(ModelState.errors[2].propertyName, 'e');
    expect(ModelState.errors[2].error, 'La propriété "e" doit être supérieure ou égale à la propriété "f"');
  });

  /*****************************************
   * Greater than
  *****************************************/
  test('Test for GreaterThan is a success. Model is valid.', () {
    GreaterThanTest tester = new GreaterThanTest(DateTime.parse('2019-01-10'), DateTime.parse('2019-01-09'), 6, 5, 7.5, 7.0);
    bool isValid = ModelState.isValid<GreaterThanTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for GreaterThan is a success. Model is not valid.', () {
    GreaterThanTest tester = new GreaterThanTest(DateTime.parse('2019-01-10'), DateTime.parse('2019-01-10'), 5, 5, 7.0, 7.0);
    bool isValid = ModelState.isValid<GreaterThanTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, GreaterThan);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être supérieure à la propriété "b"');
    expect(ModelState.errors[1].validatorType, GreaterThan);
    expect(ModelState.errors[1].propertyName, 'c');
    expect(ModelState.errors[1].error, 'La propriété "c" doit être supérieure à la propriété "d"');
    expect(ModelState.errors[2].validatorType, GreaterThan);
    expect(ModelState.errors[2].propertyName, 'e');
    expect(ModelState.errors[2].error, 'La propriété "e" doit être supérieure à la propriété "f"');
  });

  /*****************************************
   * MembershipPassword
  *****************************************/
  test('Test for MembershipPassword is a success. Model is valid.', () {
    MembershipPasswordTest tester = new MembershipPasswordTest('0Siatis@0719');
    bool isValid = ModelState.isValid<MembershipPasswordTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for MembershipPassword is a success. Model is not valid.', () {
    MembershipPasswordTest tester = new MembershipPasswordTest('0Siatis0719');
    bool isValid = ModelState.isValid<MembershipPasswordTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, MembershipPassword);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'Le mot de passe le respecte pas la nomenclature');
  });

  /*****************************************
   * Range
  *****************************************/
  test('Test for Range is a success. Model is valid.', () {
    RangeTest tester = new RangeTest(75, DateTime.parse('2019-09-09'));
    bool isValid = ModelState.isValid<RangeTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for Range is a success. Model is not valid.', () {
    RangeTest tester = new RangeTest(25, DateTime.parse('2018-09-09'));
    bool isValid = ModelState.isValid<RangeTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, NumberRange);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être comprise entre 50 et 100');
    expect(ModelState.errors[1].validatorType, DateTimeRange);
    expect(ModelState.errors[1].propertyName, 'b');
    expect(ModelState.errors[1].error, 'La propriété "b" doit être comprise entre 01/01/2019 et le 31/12/2019');
  });

  /*****************************************
   * Regular expression
  *****************************************/
  test('Test for RegularExpression is a success. Model is valid.', () {
    RegularExpressionTest tester = new RegularExpressionTest('Hello');
    bool isValid = ModelState.isValid<RegularExpressionTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });
  
  test('Test for RegularExpression is a success. Model is not valid.', () {
    RegularExpressionTest tester = new RegularExpressionTest('Good morning');
    bool isValid = ModelState.isValid<RegularExpressionTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, RegularExpression);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'Le format attendu est mauvais');
  });

  /*****************************************
   * Required
  *****************************************/
  test('Test for Required is a success. Model is valid.', () {
    RequiredTest tester = new RequiredTest('value a', 'value b');
    bool isValid = ModelState.isValid<RequiredTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for Required is a failure. Model is not valid.', () {
    RequiredTest tester = new RequiredTest(null, 'value a');
    bool isValid = ModelState.isValid<RequiredTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors.first.propertyName, 'a');
    expect(ModelState.errors.first.validatorType, Required);
    expect(ModelState.errors.first.error, 'La propriété "a" est requise');
  });

  /*****************************************
   * Smaller or equal to
  *****************************************/
  test('Test for SmallerOrEqualTo is a success. Model is valid.', () {
    SmallerOrEqualToTest tester = new SmallerOrEqualToTest(DateTime.parse('2019-01-09'), DateTime.parse('2019-01-10'), 5, 5, 7.0, 7.5);
    bool isValid = ModelState.isValid<SmallerOrEqualToTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for SmallerOrEqualTo is a success. Model is not valid.', () {
    SmallerOrEqualToTest tester = new SmallerOrEqualToTest(DateTime.parse('2019-01-10'), DateTime.parse('2019-01-09'), 6, 5, 7.1, 7.0);
    bool isValid = ModelState.isValid<SmallerOrEqualToTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, SmallerOrEqualTo);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être inférieure ou égale à la propriété "b"');
    expect(ModelState.errors[1].validatorType, SmallerOrEqualTo);
    expect(ModelState.errors[1].propertyName, 'c');
    expect(ModelState.errors[1].error, 'La propriété "c" doit être inférieure ou égale à la propriété "d"');
    expect(ModelState.errors[2].validatorType, SmallerOrEqualTo);
    expect(ModelState.errors[2].propertyName, 'e');
    expect(ModelState.errors[2].error, 'La propriété "e" doit être inférieure ou égale à la propriété "f"');
  });

  /*****************************************
   * Smaller than
  *****************************************/
  test('Test for SmallerThan is a success. Model is valid.', () {
    SmallerThanTest tester = new SmallerThanTest(DateTime.parse('2019-01-09'), DateTime.parse('2019-01-10'), 5, 6, 7.0, 7.5);
    bool isValid = ModelState.isValid<SmallerThanTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for SmallerThan is a success. Model is not valid.', () {
    SmallerThanTest tester = new SmallerThanTest(DateTime.parse('2019-01-10'), DateTime.parse('2019-01-10'), 5, 5, 7.0, 7.0);
    bool isValid = ModelState.isValid<SmallerThanTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, SmallerThan);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit être inférieure à la propriété "b"');
    expect(ModelState.errors[1].validatorType, SmallerThan);
    expect(ModelState.errors[1].propertyName, 'c');
    expect(ModelState.errors[1].error, 'La propriété "c" doit être inférieure à la propriété "d"');
    expect(ModelState.errors[2].validatorType, SmallerThan);
    expect(ModelState.errors[2].propertyName, 'e');
    expect(ModelState.errors[2].error, 'La propriété "e" doit être inférieure à la propriété "f"');
  });

  /*****************************************
   * String length
  *****************************************/
  test('Test for StringLength is a success. Model is valid.', () {
    StringLengthTest tester = new StringLengthTest('Hello everybody');
    bool isValid = ModelState.isValid<StringLengthTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for StringLength is a success. Model is not valid.', () {
    StringLengthTest tester = new StringLengthTest('No, I am your father !');
    bool isValid = ModelState.isValid<StringLengthTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, StringLength);
    expect(ModelState.errors[0].propertyName, 'a');
    expect(ModelState.errors[0].error, 'La propriété "a" doit avoir une longueur entre 10 et 20 caractères');
  });

  /*****************************************
   * Multiple validation
  *****************************************/
  test('Test for multiple validation is a success. Model is valid.', () {
    MultipleValidationTest tester = new MultipleValidationTest('Maxime', 'AUBRY', 'M', DateTime.parse('1986-12-22'), null);
    bool isValid = ModelState.isValid<MultipleValidationTest>(tester);
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for multiple validation is a success. Model is not valid.', () {
    MultipleValidationTest tester = new MultipleValidationTest(null, 'azertyuiopqsdfghjklmwxcvbnazertyuiopqsdfghjklmwxcvbn', '', DateTime.parse('1986-12-22'), DateTime.parse('1986-12-21'));
    bool isValid = ModelState.isValid<MultipleValidationTest>(tester);
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, Required);
    expect(ModelState.errors[0].propertyName, 'firstname');
    expect(ModelState.errors[0].error, 'Firstname is required');
    expect(ModelState.errors[1].validatorType, StringLength);
    expect(ModelState.errors[1].propertyName, 'lastname');
    expect(ModelState.errors[1].error, 'Lastname must have between 3 and 32 characters');
    expect(ModelState.errors[2].validatorType, StringLength);
    expect(ModelState.errors[2].propertyName, 'gender');
    expect(ModelState.errors[2].error, 'Gender must have 1 character');
    expect(ModelState.errors[3].validatorType, GreaterOrEqualTo);
    expect(ModelState.errors[3].propertyName, 'dateOfDeath');
    expect(ModelState.errors[3].error, 'DateOfDeath must be greater than Birthday');
  });

  /*****************************************
   * Validation on one only property
  *****************************************/
  test('Test for validation on one only property. Property is valid.', () {
    MultipleValidationTest tester = new MultipleValidationTest('Maxime', 'AUBRY', 'M', DateTime.parse('1986-12-22'), null);
    bool isValid = ModelState.isValidProperty<MultipleValidationTest>(tester, 'firstname');
    expect(isValid, true);
    expect(ModelState.errors.isEmpty, true);
  });

  test('Test for validation on one only property. Property is not valid.', () {
    MultipleValidationTest tester = new MultipleValidationTest(null, 'AUBRY', 'M', DateTime.parse('1986-12-22'), null);
    bool isValid = ModelState.isValidProperty<MultipleValidationTest>(tester, 'firstname');
    expect(isValid, false);
    expect(ModelState.errors[0].validatorType, Required);
    expect(ModelState.errors[0].propertyName, 'firstname');
    expect(ModelState.errors[0].error, 'Firstname is required');
  });
}

@easyValidator
class EqualToTest {
  EqualToTest(this.a, this.b, this.c, this.d, this.e, this.f, this.g, this.h);
  
  @EqualTo(propertyName: 'b', error: 'La propriété "a" doit être égale à la propriété "b"')
  final DateTime a;
  final DateTime b;

  @EqualTo(propertyName: 'd', error: 'La propriété "c" doit être égale à la propriété "d"')
  final int c;
  final int d;

  @EqualTo(propertyName: 'f', error: 'La propriété "e" doit être égale à la propriété "f"')
  final double e;
  final double f;

  @EqualTo(propertyName: 'h', error: 'La propriété "g" doit être égale à la propriété "h"')
  final String g;
  final String h;
}

@easyValidator
class GreaterOrEqualToTest {
  GreaterOrEqualToTest(this.a, this.b, this.c, this.d, this.e, this.f);

  @GreaterOrEqualTo(propertyName: 'b', error: 'La propriété "a" doit être supérieure ou égale à la propriété "b"')
  final DateTime a;
  final DateTime b;

  @GreaterOrEqualTo(propertyName: 'd', error: 'La propriété "c" doit être supérieure ou égale à la propriété "d"')
  final int c;
  final int d;

  @GreaterOrEqualTo(propertyName: 'f', error: 'La propriété "e" doit être supérieure ou égale à la propriété "f"')
  final double e;
  final double f;
}

@easyValidator
class GreaterThanTest {
  GreaterThanTest(this.a, this.b, this.c, this.d, this.e, this.f);
  
  @GreaterThan(propertyName: 'b', error: 'La propriété "a" doit être supérieure à la propriété "b"')
  final DateTime a;
  final DateTime b;

  @GreaterThan(propertyName: 'd', error: 'La propriété "c" doit être supérieure à la propriété "d"')
  final int c;
  final int d;

  @GreaterThan(propertyName: 'f', error: 'La propriété "e" doit être supérieure à la propriété "f"')
  final double e;
  final double f;
}

@easyValidator
class MembershipPasswordTest {
  MembershipPasswordTest(this.a);
  
  @MembershipPassword(minLength: 8, maxLength: 16, includesAlphabeticalCharacters: true, includesUppercaseCharacters: true, includesNumericalCharacters: true, includesSpecialCharacters: true, error: 'Le mot de passe le respecte pas la nomenclature')
  final String a;
}

@easyValidator
class RangeTest {
  RangeTest(this.a, this.b);

  @NumberRange(min: 50, max: 100, error: 'La propriété "a" doit être comprise entre 50 et 100')
  final int a;
  
  @DateTimeRange(min: '2019-01-01', max: null, error: 'La propriété "b" doit être comprise entre 01/01/2019 et le 31/12/2019')
  final DateTime b;
}

@easyValidator
class RegularExpressionTest {
  RegularExpressionTest(this.a);

  @RegularExpression(expression: r'^Hello|Bye$', error: 'Le format attendu est mauvais')
  final String a;
}

@easyValidator
class RequiredTest {
  RequiredTest(this.a, this.b);
  
  @Required(error: 'La propriété "a" est requise')
  final String a;
  final String b;
}

@easyValidator
class SmallerOrEqualToTest {
  SmallerOrEqualToTest(this.a, this.b, this.c, this.d, this.e, this.f);

  @SmallerOrEqualTo(propertyName: 'b', error: 'La propriété "a" doit être inférieure ou égale à la propriété "b"')
  final DateTime a;
  final DateTime b;

  @SmallerOrEqualTo(propertyName: 'd', error: 'La propriété "c" doit être inférieure ou égale à la propriété "d"')
  final int c;
  final int d;

  @SmallerOrEqualTo(propertyName: 'f', error: 'La propriété "e" doit être inférieure ou égale à la propriété "f"')
  final double e;
  final double f;
}

@easyValidator
class SmallerThanTest {
  SmallerThanTest(this.a, this.b, this.c, this.d, this.e, this.f);

  @SmallerThan(propertyName: 'b', error: 'La propriété "a" doit être inférieure à la propriété "b"')
  final DateTime a;
  final DateTime b;

  @SmallerThan(propertyName: 'd', error: 'La propriété "c" doit être inférieure à la propriété "d"')
  final int c;
  final int d;

  @SmallerThan(propertyName: 'f', error: 'La propriété "e" doit être inférieure à la propriété "f"')
  final double e;
  final double f;
}

@easyValidator
class StringLengthTest {
  StringLengthTest(this.a);
  
  @StringLength(min: 10, max: 20, error: 'La propriété "a" doit avoir une longueur entre 10 et 20 caractères')
  final String a;
}

@easyValidator
class MultipleValidationTest {
  MultipleValidationTest(this.firstname, this.lastname, this.gender, this.birthday, this.dateOfDeath);
  
  @Required(error: 'Firstname is required')
  @StringLength(min: 3, max: 32, error: 'Firstname must have between 3 and 32 characters')
  final String firstname;
  
  @Required(error: 'Lastname is required')
  @StringLength(min: 3, max: 32, error: 'Lastname must have between 3 and 32 characters')
  final String lastname;

  @Required(error: 'Gender is required')
  @StringLength(min: 1, max: 1, error: 'Gender must have 1 character')
  final String gender;

  @Required(error: 'Birthday is requried')
  @DateTimeRange(min: '1900-01-01', max: null, error: 'Birthday must be betwwen 1900/01/01 and infinity')
  final DateTime birthday;

  @DateTimeRange(min: '1900-01-01', max: null, error: 'DateOfDeath must be betwwen 1900/01/01 and infinity')
  @GreaterOrEqualTo(propertyName: 'birthday', error: 'DateOfDeath must be greater than Birthday')
  final DateTime dateOfDeath;
}
